@@ -11,29 +11,30 @@ an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express o
 specific language governing permissions and limitations under the License.
 """
 
-from datetime import datetime
 from abc import ABCMeta, abstractmethod
-from typing import List, Optional, Dict, Set, Any, Tuple
+from datetime import datetime
+from typing import Any, Dict, List, Optional, Set, Tuple
 
 from .models import (
-    ScheduleInterruptPoint,
-    State,
-    Node,
-    Schedule,
-    ScheduleType,
+    CallbackData,
+    ContextValue,
     Data,
     DataInput,
+    DispatchProcess,
+    ExecuteInterruptEvent,
+    ExecuteInterruptPoint,
     ExecutionData,
     ExecutionHistory,
     ExecutionShortHistory,
-    CallbackData,
+    HookType,
+    Node,
     ProcessInfo,
-    SuspendedProcessInfo,
-    DispatchProcess,
-    ContextValue,
-    ExecuteInterruptPoint,
-    ExecuteInterruptEvent,
+    Schedule,
     ScheduleInterruptEvent,
+    ScheduleInterruptPoint,
+    ScheduleType,
+    State,
+    SuspendedProcessInfo,
 )
 
 # plugin interface
@@ -86,6 +87,28 @@ class Service(metaclass=ABCMeta):
         :rtype: bool
         """
 
+    @abstractmethod
+    def hook_dispatch(
+        self,
+        hook: HookType,
+        data: ExecutionData,
+        root_pipeline_data: ExecutionData,
+        callback_data: Optional[CallbackData] = None,
+    ) -> bool:
+        """
+        hook 分发逻辑
+        :param hook: 钩子名称
+        :type hook: HookType
+        :param data: 节点执行数据
+        :type data: ExecutionData
+        :param root_pipeline_data: 根流程执行数据
+        :type root_pipeline_data: ExecutionData
+        :param callback_data: 回调数据, defaults to None
+        :type callback_data: Optional[CallbackData], optional
+        :return: [description]
+        :rtype: bool
+        """
+
     @abstractmethod
     def need_schedule(self) -> bool:
         """
@@ -95,6 +118,13 @@ class Service(metaclass=ABCMeta):
         :rtype: bool
         """
 
+    @abstractmethod
+    def need_run_hook(self) -> bool:
+        """
+        服务是否需要执行 Hook
+        :return:
+        """
+
     @abstractmethod
     def schedule_type(self) -> Optional[ScheduleType]:
         """
@@ -184,7 +214,7 @@ class PluginManagerMixin:
     """
 
     @abstractmethod
-    def get_service(self, code: str, version: str) -> Service:
+    def get_service(self, code: str, version: str, name: str = None) -> Service:
         """
         根据代号与版本获取特定服务对象实例
 
@@ -192,6 +222,8 @@ class PluginManagerMixin:
         :type code: str
         :param version: 服务版本
         :type version: str
+        :param name: 服务名
+        :type version: str
         :return: 服务对象实例
         :rtype: Service
         """
@@ -231,6 +263,74 @@ class PluginManagerMixin:
         """
 
 
+class EngineRuntimeHooksMixin:
+    def node_execute_exception(self, root_pipeline_id: str, node_id: str, ex_data: str):
+        """
+        节点 execute 方法异常需要执行的钩子
+        :param root_pipeline_id: 任务ID
+        :type node_id: str
+        :param node_id: 节点ID
+        :type node_id: str
+        :param ex_data: 异常信息
+        :type ex_data: str
+        """
+
+    def node_schedule_exception(self, root_pipeline_id: str, node_id: str, ex_data: str):
+        """
+        节点 schedule 方法异常需要执行的钩子
+        :param root_pipeline_id: 任务ID
+        :type node_id: str
+        :param node_id: 节点ID
+        :type node_id: str
+        :param ex_data: 异常信息
+        :type ex_data: str
+        """
+
+    def node_execute_fail(self, root_pipeline_id: str, node_id: str):
+        """
+        节点 execute 方法失败需要执行的钩子
+        :param root_pipeline_id: 任务ID
+        :type node_id: str
+        :param node_id: 节点ID
+        :type node_id: str
+        """
+
+    def node_schedule_fail(self, root_pipeline_id: str, node_id: str):
+        """
+        节点 schedule 方法失败需要执行的钩子
+        :param root_pipeline_id: 任务ID
+        :type node_id: str
+        :param node_id: 节点ID
+        :type node_id: str
+        """
+
+    def node_enter(self, root_pipeline_id: str, node_id: str):
+        """
+        进入节点前
+        :param root_pipeline_id: 任务ID
+        :type node_id: str
+        :param node_id: 节点ID
+        :type node_id: str
+        """
+
+    def node_finish(self, root_pipeline_id: str, node_id: str):
+        """
+        离开节点需要执行的钩子
+        :param root_pipeline_id: 任务ID
+        :type node_id: str
+        :param node_id: 节点ID
+        :type node_id: str
+        """
+
+    def pipeline_finish(self, pipeline_id: str):
+        """
+        pipeline 结束执行的钩子
+
+        :param pipeline_id: 流程 ID
+        :type pipeline_id: str
+        """
+
+
 class EngineAPIHooksMixin:
     """
     引擎 API 执行时调用的钩子相关接口声明
@@ -1469,9 +1569,18 @@ class EventMixin:
         """
 
 
+class ConfigMixin:
+    @abstractmethod
+    def get_config(self, name):
+        """
+        获取配置
+        """
+
+
 class EngineRuntimeInterface(
     PluginManagerMixin,
     EngineAPIHooksMixin,
+    EngineRuntimeHooksMixin,
     TaskMixin,
     ProcessMixin,
     StateMixin,
@@ -1481,6 +1590,7 @@ class EngineRuntimeInterface(
     DataMixin,
     ExecutionHistoryMixin,
     InterruptMixin,
+    ConfigMixin,
     metaclass=ABCMeta,
 ):
     @abstractmethod
